local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local LCo = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Config")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

local CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName = CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName
local pairs, type = pairs, type

local raidModeValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
}

local optMGetter, optMSetter, optRMGetter, optRMSetter, optQGetter, optQSetter, optWQLSetter, optCCFSetter, optTagSetter, optHideBTr, specialItemMacro, hideCustomColorMode, zeroColorGetter, zeroColorSetter, halfColorGetter, halfColorSetter, fullColorGetter, fullColorSetter, setDefaultColor, setCustomColorMode, getCustomColorMode, hideRaidMode
do
	function optMGetter(info)
		return core:getModuleEnabled(info[#info])
	end
	function optMSetter(info, value)
		core:setModuleEnabled(info[#info], value)
		if info[#info] == "questsTracker" then
			if not value then
				core.cbh:Fire("questsTrackerDisabled")
			else
				core.cbh:Fire("questsTrackerEnabled")
			end
		end
	end
	function optRMGetter(info)
		return core.db.profile.raidModules[info[#info]]
	end
	function optRMSetter(info, value)
		core.db.profile.raidModules[info[#info]] = value
		core:updateRaidMode(value)
	end
	function optQGetter(info)
		return core.db.profile.quests[info[#info]]
	end
	function optQSetter(info, value)
		core.db.profile.quests[info[#info]] = value
	end
	function optWQLSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestLevel()
	end
	function optCCFSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestsObjectivesStatus()
	end
	function optTagSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestTag()
	end
	function optHideBTr(info, value)
		core.db.profile.quests[info[#info]] = value
		core:hideShowDefaultBlizTracker()
	end
	function specialItemMacro(info, value)
		local index = GetMacroIndexByName("tomQuest2_SI")
		if index == 0 then
			index = CreateMacro("tomQuest2_SI", 1, "", 0)
			PickupMacro(index)
		else
			EditMacro(index, "tomQuest2_SI", 1, "")
		end
	end
	function hideCustomColorMode()
		if core.db.profile.quests.customColorMode then
			return true
		else
			return false
		end
	end
	function normalizeColorComponent(component)
		if component > 1 then return 1 elseif component < 0 then return 0 else return component end
	end
	function zeroColorGetter(info)
		return normalizeColorComponent(core.db.profile.quests.rj0) , normalizeColorComponent(core.db.profile.quests.gj0), normalizeColorComponent(core.db.profile.quests.bj0)
	end
	function halfColorGetter(info)
		return normalizeColorComponent(0.5*core.db.profile.quests.ri1 + core.db.profile.quests.rj1), normalizeColorComponent(0.5*core.db.profile.quests.gi1 + core.db.profile.quests.gj1), normalizeColorComponent(0.5*core.db.profile.quests.bi1 + core.db.profile.quests.bj1)
	end
	function fullColorGetter(info)
		return normalizeColorComponent(core.db.profile.quests.rj3), normalizeColorComponent(core.db.profile.quests.gj3), normalizeColorComponent(core.db.profile.quests.bj3)
	end
	function computeFirstFunction(r0, g0, b0, rh, gh, bh)
		core.db.profile.quests.rj1 = r0
		core.db.profile.quests.gj1 = g0
		core.db.profile.quests.bj1 = b0
		core.db.profile.quests.ri1 = 2 * (rh - r0)
		core.db.profile.quests.gi1 = 2 * (gh - g0)
		core.db.profile.quests.bi1 = 2 * (bh - b0)
	end
	function computeSecondFunction(rh, gh, bh, r1, g1, b1)
		core.db.profile.quests.rj2 = 2 * rh - r1
		core.db.profile.quests.gj2 = 2 * gh - g1
		core.db.profile.quests.bj2 = 2 * bh - b1
		core.db.profile.quests.ri2 = 2 * (r1 - rh)
		core.db.profile.quests.gi2 = 2 * (g1 - gh)
		core.db.profile.quests.bi2 = 2 * (b1 - bh)
	end
	function zeroColorSetter(info, arg1, arg2, arg3)
		core.db.profile.quests.rj0 = arg1
		core.db.profile.quests.gj0 = arg2
		core.db.profile.quests.bj0 = arg3
		local rh, gh, bh = halfColorGetter()
		computeFirstFunction(arg1, arg2, arg3, rh, gh, bh)
		core:computeQuestsObjectivesStatus()
	end
	function halfColorSetter(info, arg1, arg2, arg3)
		local r0, g0, b0 = zeroColorGetter()
		computeFirstFunction(r0, g0, b0, arg1, arg2, arg3)
		local r1, g1, b1 = fullColorGetter()
		computeSecondFunction(arg1, arg2, arg3, r1, g1, b1)
		core:computeQuestsObjectivesStatus()
	end
	function fullColorSetter(info, arg1, arg2, arg3)
		local rh, gh, bh = halfColorGetter()
		computeSecondFunction(rh, gh, bh, arg1, arg2, arg3)
		core.db.profile.quests.rj3 = arg1
		core.db.profile.quests.gj3 = arg2
		core.db.profile.quests.bj3 = arg3
		core:computeQuestsObjectivesStatus()
	end
	function setDefaultColor()
		core.db.profile.quests.rj0, core.db.profile.quests.gj0, core.db.profile.quests.bj0 = 1, 0, 0
		core.db.profile.quests.ri1, core.db.profile.quests.rj1, core.db.profile.quests.gi1, core.db.profile.quests.gj1, core.db.profile.quests.bi1, core.db.profile.quests.bj1 = 0, 1, 2, 0, 0 ,0
		core.db.profile.quests.ri2, core.db.profile.quests.rj2, core.db.profile.quests.gi2, core.db.profile.quests.gj2, core.db.profile.quests.bi2, core.db.profile.quests.bj2 = -2, 2, 0, 1, 0, 0
		core.db.profile.quests.rj3, core.db.profile.quests.gj3, core.db.profile.quests.bj3 = 0, 1, 0
		core:computeQuestsObjectivesStatus()
	end
	function hideRaidMode()
		if core.db.profile.raidModules.enabled then
			return false
		else
			return true
		end
	end
end

local options, moduleOptions = nil, {}
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = "tomQuest2",
			args = {
				modules = {
					type = "group",
					name = LCo["Modules"],
					desc = LCo["Modules"],
					get = optQGetter,
					args = {
						header = {
							type = "header",
							name = LCo["Modules"],
							order = 0,
						},
						raidMode = {
							type = "header",
							name = LCo["Raid mode"],
							order = 30,
						},
						enabled = {
							type = "toggle",
							name = LCo["Raid mode"],
							get = optRMGetter,
							set = optRMSetter,
							width = "full",
							order = 31,
						},
						raidModeDesc = {
							type = "description",
							name = LCo["Define module state when in a raid"],
							hidden = hideRaidMode,
							order = 32,
						},
						afterRaidHeader = {
							type = "header",
							name = "",
							order = 99,
						},
						hideDefaultBlizTracker = {
							name = LCo["Hide default blizzard quests and achievement tracker"],
							type= "toggle",
							set = optHideBTr,
							order = 100,
							width = "full",
						},
						weightedLevel = {
							type = "header",
							name = LCo["Weighted quest level computation"],
							order = 101,
						},
						eliteAddLevel = {
							name = LCo["Level to add for an elite quest"],
							type = "range",
							min = 0,
							max = 5,
							step = 1,
							set = optWQLSetter,
							order = 102,
						},
						groupAddLevel = {
							name = LCo["Level to add per missing player for a group quest"],
							type = "range",
							min = 0,
							max = 5,
							step = 1,
							set = optWQLSetter,
							order = 103,
						},
						dungeonAddLevel = {
							name = LCo["Level to add per missing player for a dungeon quest"],
							type = "range",
							min = 0,
							max = 5,
							step = 1,
							set = optWQLSetter,
							order = 104,
						},
						heroicAddLevel = {
							name = LCo["Level to add per missing player for an heroic quest"],
							type = "range",
							min = 0,
							max = 5,
							step = 1,
							set = optWQLSetter,
							order = 105,
						},
						questsTag = {
							type = "header",
							name = LCo["Custom quests tag"],
							order = 106,
						},
						elite = {
							name = LCo["Elite quest tag"],
							type = "input",
							set = optTagSetter,
							order = 107,
						},
						group = {
							name = LCo["Group quest tag"],
							type = "input",
							set = optTagSetter,
							order = 108,
						},
						pvp = {
							name = LCo["PVP quest tag"],
							type = "input",
							set = optTagSetter,
							order = 109,
						},
						raid = {
							name = LCo["Raid quest tag"],
							type = "input",
							set = optTagSetter,
							order = 110,
						},
						dungeon = {
							name = LCo["Dungeon quest tag"],
							type = "input",
							set = optTagSetter,
							order = 111,
						},
						heroic = {
							name = LCo["Heroic quest tag"],
							type = "input",
							set = optTagSetter,
							order = 112,
						},
						daily = {
							name = LCo["Daily quest tag"],
							type = "input",
							set = optTagSetter,
							order = 113,
						},
						macroHeader = {
							type = "header",
							name = LCo["Special quests item macro"],
							order = 114,
						},
						macro = {
							name = LCo["Macro Generator"],
							type = "execute",
							func = specialItemMacro,
							order = 115,
						},
						customColorFunctionHeader = {
							type = "header",
							name = LCo["Custom objective coloring function"],
							order = 116,
						},
						easyColorSetupDesc = {
							type = "description",
							name = LCo["You can define a color for percentage of completion = 0, 50% and 100%. tomQuest2 will automatically define the needed color function between those threee points"],
							order = 117,
						},
						zeroColor = {
							name = LCo["Color for completion = 0%"],
							type = "color",
							hasAlpha = true,
							get = zeroColorGetter,
							set = zeroColorSetter,
							width = "full",
							order = 118,
						},
						halfColor = {
							name = LCo["Color for completion = 50%"],
							type = "color",
							hasAlpha = true,
							get = halfColorGetter,
							set = halfColorSetter,
							width = "full",
							order = 119,
						},
						fullColor = {
							name = LCo["Color for completion = 100%"],
							type = "color",
							hasAlpha = true,
							get = fullColorGetter,
							set = fullColorSetter,
							width = "full",
							order = 120,
						},
						resetColor = {
							name = LCo["Reset color functions"],
							type = "execute",
							func = setDefaultColor,
							width = full,
							order = 121,
						},
						customColorMode = {
							name = LCo["Hide custom color function setup"],
							type = "toggle",
							width = "full",
							get = optQGetter,
							set = optQSetter,
							order = 122,
						},
						customColorFunctionDesc = {
							type = "description",
							name = LCo["Custom objective coloring allow you to define multiplicative and additive component of the coloring function. For each color component you'll have a function that is color = (multiplicative component) * percent + (additive component) where percent is the percent of completion of the objective."],
							hidden = hideCustomColorMode,
							order = 123,
						},
						zeroCustomColorFunctionHeader = {
							type = "header",
							name = LCo["Function for objective completion at 0%"],
							hidden = hideCustomColorMode,
							order = 124,
						},
						rj0 = {
							name = LCo["Red add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 125,
						},
						gj0 = {
							name = LCo["Green add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 126,
						},
						bj0 = {
							name = LCo["Blue add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 127
						},
						firstCustomColorFunctionHeader = {
							type = "header",
							name = LCo["Function for [0%, 50%[ objective completion"],
							hidden = hideCustomColorMode,
							order = 128,
						},
						ri1 = {
							name = LCo["Red mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 129,
						},
						gi1 = {
							name = LCo["Green mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 130,
						},
						bi1 = {
							name = LCo["Blue mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 131,
						},
						rj1 = {
							name = LCo["Red add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 132,
						},
						gj1 = {
							name = LCo["Green add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 133,
						},
						bj1 = {
							name = LCo["Blue add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 134,
						},
						secondCustomColorFunctionHeader = {
							type = "header",
							name = LCo["Function for ]50%, 100%[ objective completion"],
							hidden = hideCustomColorMode,
							order = 135,
						},
						ri2 = {
							name = LCo["Red mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 136,
						},
						gi2 = {
							name = LCo["Green mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 137,
						},
						bi2 = {
							name = LCo["Blue mult."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 138,
						},
						rj2 = {
							name = LCo["Red add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 139,
						},
						gj2 = {
							name = LCo["Green add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 140,
						},
						bj2 = {
							name = LCo["Blue add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 141,
						},
						fullCustomColorFunctionHeader = {
							type = "header",
							name = LCo["Function for objective completion at 100%"],
							hidden = hideCustomColorMode,
							order = 142,
						},
						rj3 = {
							name = LCo["Red add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 143,
						},
						gj3 = {
							name = LCo["Green add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 144,
						},
						bj3 = {
							name = LCo["Blue add."],
							type = "range",
							min = -5,
							max = 5,
							step = 0.1,
							set = optCCFSetter,
							hidden = hideCustomColorMode,
							order = 145,
						},
					},
				},
			},
		}
		local moduleIndex = 0
		for k, v in core:IterateModules() do
			moduleIndex = moduleIndex + 1
			options.args.modules.args[k] = {
				type = "toggle",
				name = v.modName,
				get = optMGetter,
				set = optMSetter,
				width = "double",
				desc = v.modName,
				order = moduleIndex,
			}
		end
		local raidModuleIndex = 0
		for k, v in core:IterateModules() do
			raidModuleIndex = raidModuleIndex + 1
			options.args.modules.args["raid"..k] = {
				type = "select",
				values = raidModeValues,
				name = v.modName,
				get = optRMGetter,
				set = optRMSetter,
				hidden = hideRaidMode,
				width = "full",
				desc = v.modName,
				order = raidModuleIndex + 32,
			}
		end
		for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v
		end
	end
	return options
end

function core:SetupOptions()
	InterfaceOptionsFrame:SetFrameStrata("DIALOG") 
	core.optionsFrames = {}
	ACR:RegisterOptionsTable("tomQuest2", getOptions)
	self.optionsFrames.tomQuest2 = ACD:AddToBlizOptions("tomQuest2", "tomQuest2", nil, "modules")
	self:registerModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), LCo["Profiles"])
end

function core:registerModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl
	self.optionsFrames[name] = ACD:AddToBlizOptions("tomQuest2", displayName, "tomQuest2", name)
end