-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsTracker", "AceEvent-3.0", "AceHook-3.0", "AceSerializer-3.0")
local achievementTracker = core:GetModule("achievementTracker")

-----------------------
--- Library loading ---
-----------------------
local LQTr = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsTracker")
local LSM = LibStub("LibSharedMedia-3.0")
local QTIP = LibStub("LibQTip-1.0")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local pairs, unpack, format, type, sub, tonumber = pairs, unpack, string.format, type, string.sub, tonumber
local ToggleFrame = ToggleFrame
local SetAbandonQuest, GetAbandonQuestItems, GetAbandonQuestName = SetAbandonQuest, GetAbandonQuestItems, GetAbandonQuestName
local SelectQuestLogEntry = SelectQuestLogEntry
local QuestLogPushQuest = QuestLogPushQuest
local MouseIsOver = MouseIsOver
local ItemRefTooltip = ItemRefTooltip
local GetQuestLogSelection = GetQuestLogSelection
local GetQuestLink = GetQuestLink
local CreateFrame, CreateFont, SetFont = CreateFrame, CreateFont, SetFont
local UnitIsDead = UnitIsDead
local GetSubZoneText, GetZoneText = GetSubZoneText, GetZoneText
local IsShiftKeyDown, IsControlKeyDown, IsAltKeyDown = IsShiftKeyDown, IsControlKeyDown, IsAltKeyDown
local ChatEdit_InsertLink = ChatEdit_InsertLink
local ShowUIPanel, HideUIPanel = ShowUIPanel, HideUIPanel

-----------------
--- Variables ---
-----------------
module.modName = LQTr["Quests Tracker"]
local prdb, chdb, options
local subZone, currentZone, oldZone = "", "", ""
local data = {true, true, true}
local numTQ, numTQ_BML = 0, 0
local anchorPosition = {[1] = "TOPLEFT", [2] = "TOP", [3] = "TOPRIGHT", [4] = "BOTTOMLEFT", [5] = "BOTTOM", [6] = "BOTTOMRIGHT"}
local autoTrack = {[1] = LQTr["Always"], [2] = LQTr["Below Max Level"], [3] = LQTr["Never"]}
local hideQuests = {[1] = LQTr["When you've completed the quest"], [2] = LQTr["When everyone in the party have completed the quest"], [3] = LQTr["Never"]}
local hideObjectives = {[1] = LQTr["When you've completed the objective"], [2] = LQTr["When everyone in the party have completed the objective"], [3] = LQTr["Never"]}
local outlines = {[""] = LQTr["None"], ["OUTLINE"] = LQTr["Outline"], ["THICKOUTLINE"] = LQTr["Thick Outline"]}
local defaultColors = {"defaultHeaderNameColor", "defaultHeaderLevelColor", "defaultQuestNameColor", "defaultQuestLevelColor", "defaultQuestObjectiveColor"}
local trackerAnchor, tracker = nil, nil
local hTrFontTQ2 = CreateFont("hTrFontTQ2")
local qTrFontTQ2 = CreateFont("qTrFontTQ2")
local qoTrFontTQ2 = CreateFont("qoTrFontTQ2")
local hideInCombat = nil
local interactivityList = {
	["chatLink"] = LQTr["Insert quest link in chat"],
	["objectiveProgress"] = LQTr["Insert quest objectives progress in chat"],
	["macroUpdate"] = LQTr["Update special quest item macro"],
	["questTooltip"] = LQTr["Display of quest tooltip"],
	["shareQuest"] = LQTr["Share quest"],
	["trackQuest"] = LQTr["Track quest"],
	["abandonQuest"] = LQTr["Abandon quest"],
	["displayQuest"] = LQTr["Diplay quest log"],
	["displayQuestInfo"] = LQTr["Display quest on map (external module needed)"],
	["displayCQIStartNPC"] = LQTr["Display of start npc in Cartographer quest Info"],
	["displayCQIEndNPC"] = LQTr["Display of end npc in Cartographer Quest Info"],
	["displayCQIObjective"] = LQTr["Display of objective in Cartographer Quest Info"],
	["displayMMObjective"] = LQTr["Display of objective in your map with MobMap"],
	["nothing"] = LQTr["Do nothing"],
}

----------------------------
--- Tracker Anchor Frame ---
----------------------------
local anchorSetPoint = {
	[1] = {point = "TOPLEFT", relPoint = "BOTTOMLEFT", x = 0, y = 24},
	[2] = {point = "TOP", relPoint = "BOTTOM", x = 0, y = 24},
	[3] = {point = "TOPRIGHT", relPoint = "BOTTOMRIGHT", x = 0, y = 24},
	[4] = {point = "BOTTOMLEFT", relPoint = "TOPLEFT", x = 0, y = -24},
	[5] = {point = "BOTTOM", relPoint = "TOP", x = 0, y = -24},
	[6] = {point = "BOTTOMRIGHT", relPoint = "TOPRIGHT", x = 0, y = -24}}

local anchor = CreateFrame("Button", tomQuest2QuestTrackerAnchor, UIParent)
anchor:SetHeight(24)
anchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5}, tile = true, tileSize = 16})
anchor:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
anchor:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
local text = anchor:CreateFontString(nil, nil, "GameFontNormalSmall")
text:SetPoint("CENTER")
text:SetText(LQTr["Tracker"])
anchor:SetWidth(text:GetStringWidth() + 8)
anchor:SetMovable(true)
anchor:RegisterForDrag("LeftButton")
anchor:SetClampedToScreen(true)
anchor:SetScript("OnDragStart", function(self)
	self:StartMoving()
end)
anchor:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
	prdb.point, prdb.x, prdb.y = "BOTTOMLEFT", self:GetLeft(), self:GetBottom()
	trackerAnchor = anchor:GetCenter() > UIParent:GetWidth()*0.5 and "ANCHOR_LEFT" or "ANCHOR_RIGHT"
	module:updateQuestsTracker()
end)

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		point = "CENTER",
		x = 0,
		y = 0,
		onLogIn = false,
		onQuestAccepted = 3,
		onObjectiveProgress = true,
		onSubZone = true,
		onZone = true,
		onDaily = true,
		onClass = true,
		showAnchor = true,
		anchorPosition = 2,
		toggleTracker = true,
		hideTrackerInCombat = true,
		hideCompletedQuest = 2,
		hideCompletedObjective = 2,
		onlyCurrentZone = false,
		limitQuestLevel = false,
		headerFont = LSM.DefaultMedia.font,
		headerFontSize = 12,
		headerFontOutline = "",
		questFont = LSM.DefaultMedia.font,
		questFontSize = 10,
		questFontOutline = "",
		questObjectiveFont = LSM.DefaultMedia.font,
		questObjectiveFontSize = 8,
		questObjectiveFontOutline = "",
		maxWidth = 300,
		hasMaxHeight = true,
		maxHeight = 600,
		scale = 1,
		showQuestItemIcon = true,
		questItemIconSize = 24,
		showItemsStartingQuest = true,
		showUsableQuestItemsNotAttached = true,
		questsCellPadding = 5,
		objectivesCellPadding = 10,
		rightCellPadding = 5,
		backDropColor = {0.09, 0.09, 0.09, 1},
		borderColor = {1, 1, 1, 1},
		colorizeQuestName = true,
		defaultQuestNameColor = "ffffff",
		colorizeQuestLevel = true,
		defaultQuestLevelColor = "ffffff",
		colorizeQuestObjective = true,
		defaultQuestObjectiveColor = "ffffff",
		colorizeQuestObjectiveCounter = true,
		displayHeader = true,
		autoCollapseHeader = true,
		collapseButtonSize = 24,
		colorizeHeaderName = true,
		defaultHeaderNameColor = "ffffff",
		hideZoneLevelRange = false,
		colorizeHeaderLevel = true,
		defaultHeaderLevelColor = "ffffff",
		disableInteraction = false,
		overTooltip = true,
		mouseRightShift = "chatLink",
		mouseRightControl = "objectiveProgress",
		mouseRightAlt = "macroUpdate",
		mouseRight = "questTooltip",
		mouseLeftShift = "trackQuest",
		mouseLeftControl = "shareQuest",
		mouseLeftAlt = "abandonQuest",
		mouseLeft = "displayQuest",
		mouseMiddleShift = "nothing",
		mouseMiddleControl = "nothing",
		mouseMiddleAlt = "nothing",
		mouseMiddle = "nothing",

	},
}

---------------
--- Options ---
---------------
local optGetter, optSetter, colorGetter, colorSetter, hexColorGetter, hexColorSetter, anchorSetter, combatSetter, maxLDGetter, maxLDSetter, headFontSetter, questFontSetter, qoFontSetter, hideHeader, hideHeader2, hideHeader3, hideHeader4, hideLimitQuestLevel, hideItemIcon, hideQNColor, hideQLColor, hideQOColor, hideMaxHeight, interactionDisabled
do
	function optGetter(info)
		return prdb[info[#info]]
	end
	function optSetter(info, value)
		prdb[info[#info]] = value
		QTIP:Release(tracker)
		tracker = nil
		module:updateQuestsTracker()
	end
	function colorGetter(info)
		return unpack(prdb[info[#info]])
	end
	function colorSetter(info, arg1, arg2, arg3, arg4)
		prdb[info[#info]][1] = arg1
		prdb[info[#info]][2] = arg2
		prdb[info[#info]][3] = arg3
		prdb[info[#info]][4] = arg4
		module:updateQuestsTracker()
	end
	function hexColorGetter(info)
		return tonumber(sub(prdb[info[#info]], 1, 2), 16) / 255, tonumber(sub(prdb[info[#info]], 3, 4), 16) / 255, tonumber(sub(prdb[info[#info]], 5, 6), 16) / 255
	end
	function hexColorSetter(info, arg1, arg2, arg3)
		prdb[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
		module:updateQuestsTracker()
	end
	function anchorSetter(info, value)
		prdb[info[#info]] = value
		if value then
			anchor:Show()
		else
			anchor:Hide()
		end
		module:updateQuestsTracker()
	end
	function combatSetter(info, value)
		prdb[info[#info]] = value
		hideInCombat = nil
		if value then
			core.RegisterCallback(module, "outOfCombat")
			core.RegisterCallback(module, "inCombat")
		else
			core.UnregisterCallback(module, "outOfCombat")
			core.UnregisterCallback(module, "inCombat")
		end
	end
	function maxLDGetter(info)
		return core.db.profile.quests[info[#info]]
	end
	function maxLDSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		module:questsWeightedLevelUpdated()
		module:updateQuestsTracker()
	end
	function headFontSetter(info, value)
		prdb[info[#info]] = value
		hTrFontTQ2:SetFont(LSM:Fetch("font", prdb.headerFont), prdb.headerFontSize, prdb.headerFontOutline)
		module:updateQuestsTracker()
	end
	function questFontSetter(info, value)
		prdb[info[#info]] = value
		qTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questFont), prdb.questFontSize, prdb.questFontOutline)
		module:updateQuestsTracker()
	end
	function qoFontSetter(info, value)
		prdb[info[#info]] = value
		qoTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questObjectiveFont), prdb.questObjectiveFontSize, prdb.questObjectiveFontOutline)
		module:updateQuestsTracker()
	end
	function hideHeader()
		if prdb.displayHeader then
			return false
		else
			return true
		end
	end
	function hideHeader2()
		if (not prdb.displayHeader) or (prdb.displayHeader and prdb.colorizeHeaderName) then
			return true
		else
			return false
		end
	end
	function hideHeader3()
		if (not prdb.displayHeader) or (prdb.displayHeader and prdb.hideZoneLevelRange) then
			return true
		else
			return false
		end
	end
	function hideHeader4()
		if (not prdb.displayHeader) or (prdb.displayHeader and (prdb.hideZoneLevelRange or (not prdb.hideZoneLevelRange and prdb.colorizeHeaderLevel))) then
			return true
		else
			return false
		end
	end
	function hideLimitQuestLevel()
		if prdb.limitQuestLevel then
			return false
		else
			return true
		end
	end
	function hideItemIcon()
		if prdb.showQuestItemIcon then
			return false
		else
			return true
		end
	end
	function hideQNColor()
		if not prdb.colorizeQuestName then
			return false
		else
			return true
		end
	end
	function hideQLColor()
		if not prdb.colorizeQuestLevel then
			return false
		else
			return true
		end
	end
	function hideQOColor()
		if not prdb.colorizeQuestObjective then
			return false
		else
			return true
		end
	end
	function hideMaxHeight()
		if prdb.hasMaxHeight then
			return false
		else
			return true
		end
	end
	function interactionDisabled()
		if prdb.disableInteraction then
			return true
		else
			return false
		end
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = optGetter,
			set = optSetter,
			args = {
				onLogIn = {
					name = LQTr["Track all quests on log in"],
					type = "toggle",
					width = "full",
					order = 0,
				},
				onQuestAccepted = {
					name = LQTr["Auto track quests when you accept them"],
					type = "select",
					values = autoTrack,
					width = "full",
					order = 1,
				},
				onObjectiveProgress = {
					name = LQTr["Auto track quests on objective progress"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				onSubZone = {
					name = LQTr["Auto track quests on sub zone change"],
					type = "toggle",
					width = "full",
					order = 3,
				},
				onZone = {
					name = LQTr["Auto track quests on zone change"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				onDaily = {
					name = LQTr["Auto track daily quests"],
					type = "toggle",
					width = "full",
					order = 5,
				},
				onClass = {
					name = LQTr["Auto track class quests"],
					type = "toggle",
					width = "full",
					order = 6,
				},
				showAnchor = {
					name = LQTr["Show quest tracker anchor"],
					type = "toggle",
					width = "full",
					set = anchorSetter,
					order = 7,
				},
				anchorPosition = {
					name = LQTr["Anchor position"],
					type = "select",
					values = anchorPosition,
					width = "full",
					order = 8,
				},
				trackerHeader = {
					type = "header",
					name = LQTr["Tracker Settings"],
					order = 12,
				},
				displayHeader = {
					name = LQTr["Display quests by zone"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				autoCollapseHeader = {
					name = LQTr["Automatically expand /collapse zone header on zone change"],
					hidden = hideHeader,
					type = "toggle",
					width = "full",
					order = 14,
				},
				toggleTracker = {
					name = LQTr["Display tracker"],
					type = "toggle",
					width = "full",
					order = 15,
				},
				hideTrackerInCombat = {
					name = LQTr["Hide quest tracker in combat"],
					type = "toggle",
					set = combatSetter,
					width = "full",
					order = 16,
				},
				hideCompletedQuest = {
					name = LQTr["Hide completed quest"],
					type = "select",
					values = hideQuests,
					width = "full",
					order = 17,
				},
				hideCompletedObjective = {
					name = LQTr["Hide completed quests Objectives"],
					type = "select",
					values = hideObjectives,
					width = "full",
					order = 18,
				},
				onlyCurrentZone = {
					name = LQTr["Hide quests that are not in the current zone"],
					type = "toggle",
					width = "full",
					order = 19,
				},
				limitQuestLevel = {
					name = LQTr["Hide high level quests (You'll have to choose the maximum level difference)"],
					type = "toggle",
					width = "full",
					order = 20,
				},
				maxLevelDiff = {
					name = LQTr["Maximum level difference between the weighted quest level and your level"],
					hidden = hideLimitQuestLevel,
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					get = maxLDGetter,
					set = maxLDSetter,
					width = "full",
					order = 21,
				},
				headerFont = {
					name = LQTr["Zone header font"],
					hidden = hideHeader,
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					set = headFontSetter,
					width = "full",
					order = 22,
				},
				headerFontSize = {
					name = LQTr["Zone header font size"],
					hidden = hideHeader,
					type = "range",
					min = 6,
					max = 20,
					step = 1,
					set = headFontSetter,
					width = "full",
					order = 23,
				},
				headerFontOutline = {
					name = LQTr["Zone header font outline"],
					type = "select",
					values = outlines,
					set = headFontSetter,
					width = "full",
					order = 24,
				},
				questFont = {
					name = LQTr["Quests font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					set = questFontSetter,
					width = "full",
					order = 25,
				},
				questFontSize = {
					name = LQTr["Quests font size"],
					type = "range",
					min = 6,
					max = 20,
					step = 1,
					set = questFontSetter,
					width = "full",
					order = 26,
				},
				questFontOutline = {
					name = LQTr["Quests font outline"],
					type = "select",
					values = outlines,
					set = questFontSetter,
					width = "full",
					order = 27,
				},
				questObjectiveFont = {
					name = LQTr["Quests objectives font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					set = qoFontSetter,
					width = "full",
					order = 28,
				},
				questObjectiveFontSize = { 
					name = LQTr["Quests objectives font size"],
					type = "range",
					min = 6,
					max = 20,
					step = 1,
					set = qoFontSetter,
					width = "full",
					order = 29,
				},
				questObjectiveFontOutline = {
					name = LQTr["Quests objectives font outline"],
					type = "select",
					values = outlines,
					set = qoFontSetter,
					width = "full",
					order = 30,
				},
				maxWidth = {
					name = LQTr["Quests Tracker column max width"],
					type = "range",
					min = 50,
					max = 600,
					step = 5,
					width = "full",
					order = 31,
				},
				hasMaxHeight = {
					name = LQTr["Set quests tracker maximum height after which it will be scrollable"],
					type = "toggle",
					width = "full",
					order = 32,
				},
				maxHeight = {
					name = LQTr["Quests tracker max height"],
					hidden = hideMaxHeight,
					type = "range",
					min = 100,
					max = 2000,
					step = 5,
					width = "full",
					order = 33,
				},
				scale = {
					name = LQTr["Quests Tracker scale"],
					type = "range",
					min = 0,
					max = 1,
					step = 0.1,
					width = "full",
					order = 34,
				},
				showQuestItemIcon = {
					name = LQTr["Show special quest item icon behind quest name"],
					type = "toggle",
					width = "full",
					order = 35,
				},
				questItemIconSize = {
					name = LQTr["Quest item icon size"],
					type = "range",
					hidden = hideItemIcon,
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 36,
				},
				showItemsStartingQuest = {
					name = LQTr["Show items starting a quest in the tracker"],
					type = "toggle",
					width = "full",
					order = 37,
				},
				showUsableQuestItemsNotAttached = {
					name = LQTr["Show usable quest item not linked to a quest"],
					type = "toggle",
					width = "full",
					order = 38,
				},
				questsCellPadding = {
					name = LQTr["Quests cell padding"],
					hidden = hideHeader,
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 39,
				},
				objectivesCellPadding = {
					name = LQTr["Quests objectives cell padding"],
					type = "range",
					min = 0,
					max = 50,
					step = 1,
					width = "full",
					order = 40,
				},
				rightCellPadding = {
					name = LQTr["Right-most cell right padding to avoid cell clipping"],
					type = "range",
					min = 0,
					max = 10,
					step = 1,
					width = "full",
					order = 41,
				},
				collapseButtonSize = {
					name = LQTr["Collapse Button Size"],
					hidden = hideHeader,
					type = "range",
					min = 10,
					max = 40,
					step = 1,
					width = "full",
					order = 42,
				},
				backDropColor = {
					name = LQTr["Quests Tracker background color"],
					type = "color",
					hasAlpha = true,
					get = colorGetter,
					set = colorSetter,
					width = "full",
					order = 43,
				},
				borderColor = {
					name = LQTr["Quests Tracker border color"],
					type = "color",
					hasAlpha = true,
					get = colorGetter,
					set = colorSetter,
					width = "full",
					order = 44,
				},
				colorizeHeaderName = {
					name = LQTr["Colorize Quests Header Name"],
					hidden = hideHeader,
					type = "toggle",
					width = "full",
					order = 45,
				},
				defaultHeaderNameColor = {
					name = LQTr["Default header name color"],
					type = "color",
					hasAlpha = false,
					hidden = hideHeader2,
					get = hexColorGetter,
					set = hexColorSetter,
					width = "full",
					order = 46,
				},
				hideZoneLevelRange = {
					name = LQTr["Hide zone level range"],
					hidden = hideHeader,
					type = "toggle",
					width = "full",
					order = 47,
				},
				colorizeHeaderLevel = {
					name = LQTr["Colorize Quests Header Level"],
					hidden = hideHeader3,
					type = "toggle",
					width = "full",
					order = 48,
				},
				defaultHeaderLevelColor = {
					name = LQTr["Default header level color"],
					type = "color",
					hidden = hideHeader4,
					hasAlpha = false,
					get = hexColorGetter,
					set = hexColorSetter,
					width = "full",
					order = 49,
				},
				colorizeQuestName = {
					name = LQTr["Colorize Quests Name"],
					type = "toggle",
					width = "full",
					order = 50,
				},
				defaultQuestNameColor = {
					name = LQTr["Default quest name color"],
					type = "color",
					hidden = hideQNColor,
					hasAlpha = false,
					get = hexColorGetter,
					set = hexColorSetter,
					width = "full",
					order = 51,
				},
				colorizeQuestLevel = {
					name = LQTr["Colorize Quests Level"],
					type = "toggle",
					width = "full",
					order = 52,
				},
				defaultQuestLevelColor = {
					name = LQTr["Default quest level color"],
					type = "color",
					hidden = hideQLColor,
					hasAlpha = false,
					get = hexColorGetter,
					set = hexColorSetter,
					width = "full",
					order = 53,
				},
				colorizeQuestObjective = {
					name = LQTr["Colorize Quests Objectives"],
					type = "toggle",
					width = "full",
					order = 54,
				},
				defaultQuestObjectiveColor = {
					name = LQTr["Default quest objective color"],
					type = "color",
					hidden = hideQOColor,
					hasAlpha = false,
					get = hexColorGetter,
					set = hexColorSetter,
					width = "full",
					order = 55,
				},
				colorizeQuestObjectiveCounter = {
					name = LQTr["Colorize quests objectives counter"],
					type = "toggle",
					width = "full",
					order = 56,
				},
				trackerInteraction = {
					type = "header",
					name = LQTr["Tracker interaction"],
					order = 57,
				},
				disableInteraction = {
					type = "toggle",
					name = LQTr["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make the it scrollable."],
					width = "full",
					order = 58,
				},
				overTooltip = {
					name = LQTr["Display quest tooltip on mouse over"],
					type = "toggle",
					width = "full",
					hidden = interactionDisabled,
					order = 59,
				},
				mouseRightShift = {
					name = LQTr["Action on right shift click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 60,
				},
				mouseRightControl = {
					name = LQTr["Action on right control click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 61,
				},
				mouseRightAlt = {
					name = LQTr["Action on right alt click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 62,
				},
				mouseRight = {
					name = LQTr["Action on right click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 63,
				},
				mouseLeftShift = {
					name = LQTr["Action on left shift click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 64,
				},
				mouseLeftControl = {
					name = LQTr["Action on left control click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 65,
				},
				mouseLeftAlt = {
					name = LQTr["Action on left alt click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 66,
				},
				mouseLeft = {
					name = LQTr["Action on left click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 67,
				},
				mouseMiddleShift = {
					name = LQTr["Action on middle shift click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 68,
				},
				mouseMiddleControl = {
					name = LQTr["Action on middle control click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 69,
				},
				mouseMiddleAlt = {
					name = LQTr["Action on middle alt click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 70,
				},
				mouseMiddle = {
					name = LQTr["Action on middle click"],
					type = "select",
					values = interactivityList,
					width = "full",
					hidden = interactionDisabled,
					order = 71,
				},
			},
		}
	end
	return options
end

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsTracker", defaults, "Default")
	prdb = self.db.profile
	chdb = self.db.char
	self:SetEnabledState(core:getModuleEnabled("questsTracker"))
	core:registerModuleOptions("questsTracker", getOptions, module.modName)
	chdb.questsTracked = chdb.questsTracked or {}
	chdb.questsSubZoneAutoTracked = chdb.questsSubZoneAutoTracked or {}
	chdb.questsZoneAutoTracked = chdb.questsZoneAutoTracked or {}
	chdb.zoneDisplay = chdb.zoneDisplay or {}
	core.RegisterCallback(self, "updateDBtoV3")
	LSM.RegisterCallback(self, "LibSharedMedia_Registered", "updateUsedMedia")
end

function module:updateDBtoV3(event)
	for k in pairs(defaultColors) do
		if type(prdb[k]) == "table" then
			prdb[k] = format("%02x%02x%02x", (prdb[k][1] or 0)*255, (prdb[k][2] or 0)*255, (prdb[k][3] or 0)*255)
		end
	end
end

function module:OnEnable()
	hTrFontTQ2:SetFont(LSM:Fetch("font", prdb.headerFont), prdb.headerFontSize, prdb.headerFontOutline)
	qTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questFont), prdb.questFontSize, prdb.questFontOutline)
	qoTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questObjectiveFont), prdb.questObjectiveFontSize, prdb.questObjectiveFontOutline)
	subZone = GetSubZoneText() or ""
	currentZone = GetZoneText() or ""
	anchor:SetPoint(prdb.point, prdb.x, prdb.y)
	if not prdb.showAnchor then anchor:Hide() else anchor:Show() end
	trackerAnchor = anchor:GetCenter() > UIParent:GetWidth()*0.5 and "ANCHOR_LEFT" or "ANCHOR_RIGHT"
	if prdb.hideTrackerInCombat then
		core.RegisterCallback(self, "outOfCombat")
		core.RegisterCallback(self, "inCombat")
	end
	self:RegisterEvent("ZONE_CHANGED", "zoneChanged")
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "zoneChanged")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "zoneChanged")
	self:SecureHook("AddQuestWatch")
	self:SecureHook("RemoveQuestWatch")
	self:SecureHook("GetNumQuestWatches")
	core.RegisterCallback(self, "initQuestsTracker")
	core.RegisterCallback(self, "endPlayerQuest")
	core.RegisterCallback(self, "endPartyQuest")
	core.RegisterCallback(self, "playerQuestComplete", "questStatusChanged")
	core.RegisterCallback(self, "playerQuestFailed", "questStatusChanged")
	core.RegisterCallback(self, "partyQuestComplete", "questStatusChanged")
	core.RegisterCallback(self, "partyQuestFailed", "questStatusChanged")
	core.RegisterCallback(self, "playerQuestObjectiveProgress", "questObjectiveProgress")
	core.RegisterCallback(self, "partyQuestObjectiveProgress", "questObjectiveProgress")
	core.RegisterCallback(self, "playerQuestLost", "questLost")
	core.RegisterCallback(self, "partyQuestLost", "questLost")
	core.RegisterCallback(self, "toggleTrackedQuest")
	core.RegisterCallback(self, "questObjectiveAdded")
	core.RegisterCallback(self, "questDescriptionAdded")
	core.RegisterCallback(self, "updateQuestsTracker")
	core.RegisterCallback(self, "questsWeightedLevelUpdated")
end

function module:OnDisable()
	anchor:Hide()
	QTIP:Release(tracker)
	tracker = nil
	core.UnregisterAllCallbacks(self)
	self:Refresh()
	core.cbh:Fire("updateAchievementTracker")
end

function module:Refresh()
	prdb = self.db.profile
	chdb = self.db.char
	if not self:IsEnabled() then return end
	chdb.questsTracked = chdb.questsTracked or {}
	chdb.questsSubZoneAutoTracked = chdb.questsSubZoneAutoTracked or {}
	chdb.questsZoneAutoTracked = chdb.questsZoneAutoTracked or {}
	chdb.zoneDisplay = chdb.zoneDisplay or {}
	hTrFontTQ2:SetFont(LSM:Fetch("font", prdb.headerFont), prdb.headerFontSize, prdb.headerFontOutline)
	qTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questFont), prdb.questFontSize, prdb.questFontOutline)
	qoTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questObjectiveFont), prdb.questObjectiveFontSize, prdb.questObjectiveFontOutline)
	anchor:SetPoint(prdb.point, prdb.x, prdb.y)
	if not prdb.showAnchor then anchor:Hide() else anchor:Show() end
	trackerAnchor = anchor:GetCenter() > UIParent:GetWidth()*0.5 and "ANCHOR_LEFT" or "ANCHOR_RIGHT"
	if prdb.hideTrackerInCombat then
		core.RegisterCallback(self, "outOfCombat")
		core.RegisterCallback(self, "inCombat")
	end
end

function module:updateUsedMedia(event, mediatype, key)
	if mediatype == "font" then
		local updateQuestTracker
		if key == prdb.headerFont then
			updateQuestTracker = true
			hTrFontTQ2:SetFont(LSM:Fetch("font", prdb.headerFont), prdb.headerFontSize, prdb.headerFontOutline)
		end
		if key == prdb.questFont then
			updateQuestTracker = true
			qTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questFont), prdb.questFontSize, prdb.questFontOutline)
		end
		if key == prdb.questObjectiveFont then
			updateQuestTracker = true
			qoTrFontTQ2:SetFont(LSM:Fetch("font", prdb.questObjectiveFont), prdb.questObjectiveFontSize, prdb.questObjectiveFontOutline)
		end
		if updateQuestTracker then
			module:updateQuestsTracker()
		end
	end
end

-------------------
--- zoneChanged ---
-------------------
function module:zoneChanged()
	if UnitIsDead("player") then return end
	local trackerNeedUpdate = false
	subZone = GetSubZoneText() or ""
	currentZone = GetZoneText() or ""
	if prdb.onZone and currentZone ~= oldZone then
		for uid in pairs(chdb.questsZoneAutoTracked) do
			if core.quests[uid] then
				trackerNeedUpdate = true
				RemoveQuestWatch(core.quests[uid].id, uid, true)
			else
				chdb.questsZoneAutoTracked[uid] = nil
			end
		end
		for uid, qData in pairs(core.quests) do
			if not chdb.questsTracked[uid] and qData.zone == currentZone then
				chdb.questsZoneAutoTracked[uid] = true
				trackerNeedUpdate = true
				AddQuestWatch(core.quests[uid].id, nil, uid, true)
			end
		end
	end
	if prdb.onSubZone and subZone ~= "" then
		for uid in pairs(chdb.questsSubZoneAutoTracked) do
			if core.quests[uid] then
				trackerNeedUpdate = true
				RemoveQuestWatch(core.quests[uid].id, uid, true)
			else
				chdb.questsSubZoneAutoTracked[uid] = nil
			end
		end
		for uid, qData in pairs(core.quests) do
			if not chdb.questsTracked[uid] and ((qData.objective and qData.objective:find(subZone, 1, true)) or (qData.description and qData.description:find(subZone, 1, true))) then
				chdb.questsSubZoneAutoTracked[uid] = true
				trackerNeedUpdate = true
				AddQuestWatch(core.quests[uid].id, nil, uid, true)
			end
		end
	end
	if prdb.displayHeader and prdb.autoCollapseHeader then
		for zone, zData in pairs(core.questsByZone) do
			if currentZone == zone then
				chdb.zoneDisplay[zone] = 1
			elseif zData.zone then
				chdb.zoneDisplay[zone] = 0
			else
				chdb.zoneDisplay[zone] = 1
			end
		end
		trackerNeedUpdate = true
	end
	if trackerNeedUpdate then
		module:updateQuestsTracker()
	end
	oldZone = currentZone
end

-------------
--- Hooks ---
-------------
function module:AddQuestWatch(id, timeToTrack, uid, batch)
	if not uid then
		uid = core.questsIdToUid[id]
		if not uid then return end
	end
	if (not chdb.questsTracked[uid]) or (chdb.questsTracked[uid] and core.quests[uid].tracked == 0) then
		numTQ = numTQ + 1
		core.numTQ_PlQC = core.numTQ_PlQC + core.quests[uid].playerComplete
		core.numTQ_PaQC = core.numTQ_PaQC + core.quests[uid].partyComplete
		chdb.questsTracked[uid] = true
		core.quests[uid].tracked = 1
		if core.quests[uid].complete[core.playerName] then
			core.questsByZone[core.quests[uid].zone].numTQ = core.questsByZone[core.quests[uid].zone].numTQ + 1
			core.questsByZone[core.quests[uid].zone].numTQ_PlQC = core.questsByZone[core.quests[uid].zone].numTQ_PlQC + core.quests[uid].playerComplete
			core.questsByZone[core.quests[uid].zone].numTQ_PaQC = core.questsByZone[core.quests[uid].zone].numTQ_PaQC + core.quests[uid].partyComplete
			if (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff)) then
				core.questsByZone[core.quests[uid].zone].numTQ_BML = core.questsByZone[core.quests[uid].zone].numTQ_BML + 1
				core.questsByZone[core.quests[uid].zone].numTQ_PlQC_BML = core.questsByZone[core.quests[uid].zone].numTQ_PlQC_BML + core.quests[uid].playerComplete
				core.questsByZone[core.quests[uid].zone].numTQ_PaQC_BML = core.questsByZone[core.quests[uid].zone].numTQ_PaQC_BML + core.quests[uid].partyComplete
				core.numTQ_PlQC_BML = core.numTQ_PlQC_BML + core.quests[uid].playerComplete
				core.numTQ_PaQC_BML = core.numTQ_PaQC_BML + core.quests[uid].partyComplete
			end
		else
			core.questsByZone[core.partyZone].numTQ = core.questsByZone[core.partyZone].numTQ + 1
			core.questsByZone[core.partyZone].numTQ_PaQC = core.questsByZone[core.partyZone].numTQ_PaQC + core.quests[uid].partyComplete
			if (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff)) then
				core.questsByZone[core.partyZone].numTQ_BML = core.questsByZone[core.partyZone].numTQ_BML + 1
				core.questsByZone[core.partyZone].numTQ_PaQC_BML = core.questsByZone[core.partyZone].numTQ_PaQC_BML + core.quests[uid].partyComplete
				core.numTQ_PaQC_BML = core.numTQ_PaQC_BML + core.quests[uid].partyComplete
			end
		end
		if not batch then
			module:updateQuestsTracker()
		end
	end
	if QuestLogFrame:IsVisible() then
		QuestLog_Update()
	end
end

function module:RemoveQuestWatch(id, uid, batch)
	if not uid then
		uid = core.questsIdToUid[id]
		if not uid then return end
	end
	if chdb.questsTracked[uid] then
		numTQ = numTQ - 1
		core.numTQ_PlQC = core.numTQ_PlQC - core.quests[uid].playerComplete
		core.numTQ_PaQC = core.numTQ_PaQC - core.quests[uid].partyComplete
		chdb.questsTracked[uid] = nil
		chdb.questsSubZoneAutoTracked[uid] = nil
		chdb.questsZoneAutoTracked[uid] = nil
		core.quests[uid].tracked = 0
		if core.quests[uid].complete[core.playerName] then
			core.questsByZone[core.quests[uid].zone].numTQ = core.questsByZone[core.quests[uid].zone].numTQ - 1
			core.questsByZone[core.quests[uid].zone].numTQ_PlQC = core.questsByZone[core.quests[uid].zone].numTQ_PlQC - core.quests[uid].playerComplete
			core.questsByZone[core.quests[uid].zone].numTQ_PaQC = core.questsByZone[core.quests[uid].zone].numTQ_PaQC - core.quests[uid].partyComplete
			if (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff)) then
				core.questsByZone[core.quests[uid].zone].numTQ_BML = core.questsByZone[core.quests[uid].zone].numTQ_BML - 1
				core.questsByZone[core.quests[uid].zone].numTQ_PlQC_BML = core.questsByZone[core.quests[uid].zone].numTQ_PlQC_BML - core.quests[uid].playerComplete
				core.questsByZone[core.quests[uid].zone].numTQ_PaQC_BML = core.questsByZone[core.quests[uid].zone].numTQ_PaQC_BML - core.quests[uid].partyComplete
				core.numTQ_PlQC_BML = core.numTQ_PlQC_BML - core.quests[uid].playerComplete
				core.numTQ_PaQC_BML = core.numTQ_PaQC_BML - core.quests[uid].partyComplete
			end
		else
			core.questsByZone[core.partyZone].numTQ = core.questsByZone[core.partyZone].numTQ - 1
			core.questsByZone[core.partyZone].numTQ_PaQC = core.questsByZone[core.partyZone].numTQ_PaQC - core.quests[uid].partyComplete
			if (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff)) then
				core.questsByZone[core.partyZone].numTQ_BML = core.questsByZone[core.partyZone].numTQ_BML - 1
				core.questsByZone[core.partyZone].numTQ_PaQC_BML = core.questsByZone[core.partyZone].numTQ_PaQC_BML - core.quests[uid].partyComplete
				core.numTQ_PaQC_BML = core.numTQ_PaQC_BML - core.quests[uid].partyComplete
			end
		end
		if not batch then
			module:updateQuestsTracker()
		end
	end
	if QuestLogFrame:IsVisible() then
		QuestLog_Update()
	end
end

function module:GetNumQuestWatches()
	return numTQ
end

function module:isQuestTracked(uid)
	if chdb.questsTracked[uid] then
		return true
	else
		return false
	end
end

-----------------
--- Callbacks ---
-----------------
function module:initQuestsTracker(event)
	for uid in pairs(chdb.questsTracked) do
		if not core.quests[uid] then
			chdb.questsTracked[uid] = nil
			chdb.questsSubZoneAutoTracked[uid] = nil
			chdb.questsZoneAutoTracked[uid] = nil
		end
	end
	module:updateQuestsTracker()
end

function module:questsWeightedLevelUpdated(event)
	numTQ_BML = 0
	core.numTQ_PlQC_BML = 0
	core.numTQ_PaQC_BML = 0
	for zone, zData in pairs(core.questsByZone) do
		core.questsByZone[zone].numTQ_BML = 0
		core.questsByZone[zone].numTQ_PlQC_BML = 0
		core.questsByZone[zone].numTQ_PaQC_BML = 0
		for uid in pairs(zData.quests) do
			if core.quests[uid].tracked == 1 and (core.quests[uid].weightedLevel - core.playerLevel) <= tonumber(core.db.profile.quests.maxLevelDiff) then
				numTQ_BML = numTQ_BML + 1
				core.numTQ_PlQC_BML = core.numTQ_PlQC_BML + core.quests[uid].playerComplete
				core.numTQ_PaQC_BML = core.numTQ_PaQC_BML + core.quests[uid].partyComplete
				core.questsByZone[zone].numTQ_BML = core.questsByZone[zone].numTQ_BML + 1
				core.questsByZone[zone].numTQ_PlQC_BML = core.questsByZone[zone].numTQ_PlQC_BML + core.quests[uid].playerComplete
				core.questsByZone[zone].numTQ_PaQC_BML = core.questsByZone[zone].numTQ_PaQC_BML + core.quests[uid].partyComplete
			end
		end
	end
	module:updateQuestsTracker()
end

function module:endPlayerQuest(event, first, commHandler, uid, started)
	local questAddedToTracker = 0
	if first then
		if core.quests[uid].tracked == 0 and (chdb.questsTracked[uid] or prdb.onLogIn) then
			AddQuestWatch(core.quests[uid].id, nil, uid, first)
		end
	elseif prdb.onQuestAccepted == 1 or (prdb.onQuestAccepted == 2 and core.playerLevel ~= MAX_PLAYER_LEVEL) or (prdb.onObjectiveProgress and started == 1) or (prdb.onDaily and core.quests[uid].daily == 1) or (prdb.onClass and core.quests[uid].class) then
		if not chdb.questsTracked[uid] then
			questAddedToTracker = 1
			AddQuestWatch(core.quests[uid].id, nil, uid, first)
		else
			chdb.questsSubZoneAutoTracked[uid] = nil
			chdb.questsZoneAutoTracked[uid] = nil
		end
	elseif prdb.onZone and core.quests[uid].zone == currentZone then
		if not chdb.questsTracked[uid] then
			chdb.questsZoneAutoTracked[uid] = true
			questAddedToTracker = 1
			AddQuestWatch(core.quests[uid].id, nil, uid, first)
		else
			chdb.questsSubZoneAutoTracked[uid] = nil
		end
	elseif prdb.onSubZone and not chdb.questsTracked[uid] and subZone ~= "" and ((core.quests[uid].objective and core.quests[uid].objective:find(subZone, 1, true)) or (core.quests[uid].description and core.quests[uid].description:find(subZone, 1, true))) then
		chdb.questsSubZoneAutoTracked[uid] = true
		questAddedToTracker = 1
		AddQuestWatch(core.quests[uid].id, nil, uid, first)
	end
	if not first and questAddedToTracker == 0 and chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:endPartyQuest(event, sender, uid, started, batch)
	local questAddedToTracker = 0
	if batch then
		if chdb.questsTracked[uid] and core.quests[uid].tracked == 0 then
			AddQuestWatch(core.quests[uid].id, nil, uid, batch)
		end
	elseif prdb.onQuestAccepted == 1 or (prdb.onQuestAccepted == 2 and core.playerLevel ~= MAX_PLAYER_LEVEL) or (prdb.onObjectiveProgress and started == 1) or (prdb.onDaily and core.quests[uid].daily == 1) then
		if not chdb.questsTracked[uid] then
			questAddedToTracker = 1
			AddQuestWatch(core.quests[uid].id, nil, uid, batch)
		else
			chdb.questsSubZoneAutoTracked[uid] = nil
			chdb.questsZoneAutoTracked[uid] = nil
		end
	end
	if not batch and questAddedToTracker == 0 and chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:questStatusChanged(event, first, commHandler, uid, sender)
	if chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:questObjectiveProgress(event, first, commHandler, uid, objective, got, nGot, sender)
	if prdb.onObjectiveProgress then
		chdb.questsSubZoneAutoTracked[uid] = nil
		chdb.questsZoneAutoTracked[uid] = nil
		if not chdb.questsTracked[uid] then
			AddQuestWatch(core.quests[uid].id, nil, uid, nil)
		else
			module:updateQuestsTracker()
		end
	elseif chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:questLost(event, first, commHandler, uid, abandoning, title, sender)
	if not core.quests[uid] and chdb.questsTracked[uid] then
		chdb.questsTracked[uid] = nil
		chdb.questsSubZoneAutoTracked[uid] = nil
		chdb.questsZoneAutoTracked[uid] = nil
		numTQ = numTQ - 1
		module:updateQuestsTracker()
	elseif core.quests[uid] and chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:toggleTrackedQuest(event, uid)
	if chdb.questsTracked[uid] then
		RemoveQuestWatch(core.quests[uid].id, uid, nil)
	elseif not chdb.questsTracked[uid] then
		AddQuestWatch(core.quests[uid].id, nil, uid, nil)
	end
end

function module:questObjectiveAdded(event, uid)
	if prdb.onSubZone and subZone ~= "" then
		if not chdb.questsTracked[uid] and core.quests[uid].objective and core.quests[uid].objective:find(subZone, 1, true) then
			chdb.questsSubZoneAutoTracked[uid] = true
			AddQuestWatch(core.quests[uid].id, nil, uid, nil)
		elseif chdb.questsTracked[uid] then
			module:updateQuestsTracker()
		end
	elseif chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:questDescriptionAdded(event, uid)
	if prdb.onSubZone and subZone ~= "" then
		if not chdb.questsTracked[uid] and core.quests[uid].description	and core.quests[uid].description:find(subZone, 1, true) then
			chdb.questsSubZoneAutoTracked[uid] = true
			AddQuestWatch(core.quests[uid].id, nil, uid, nil)
		elseif chdb.questsTracked[uid] then
			module:updateQuestsTracker()
		end
	elseif chdb.questsTracked[uid] then
		module:updateQuestsTracker()
	end
end

function module:inCombat(event)
	hideInCombat = true
	if tracker then tracker:Hide() end
end

function module:outOfCombat(event)
	hideInCombat = nil
	module:updateQuestsTracker()
end

---------------------------------------------------
--- QTIP Mouse Handler and Custom Cell Provider ---
---------------------------------------------------
local interactivity = {
	chatLink = function(value)
		if core.quests[value].complete[core.playerName] and core.quests[value].id then
			ChatFrameEditBox:Show()
			ChatEdit_InsertLink(GetQuestLink(core.quests[value].id))
		end
	end,
	objectiveProgress = function(value)
		if core.quests[value].numObjectives > 0 and core.quests[value].complete[core.playerName] then
			ChatFrameEditBox:Show()
			for objective, qoData in pairs(core.questsObjectives[value]) do
				local got, _ = qoData.got[core.playerName]:match("^(.-)~(%d-)$")
				ChatFrameEditBox:Insert(format("{%s %s/%s} ", qoData.desc, got, qoData.need))
			end
		end
	end,
	macroUpdate = function(value)
		core:generateSpecialItemMacro(value)
	end,
	questTooltip = function(value)
		if core.quests[value].complete[core.playerName] and core.quests[value].id then
			ItemRefTooltip:ClearLines()
			ShowUIPanel(ItemRefTooltip)
			if not ItemRefTooltip:IsVisible() then
				ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			end
			local link = GetQuestLink(core.quests[value].id)
			if link then
				ItemRefTooltip:SetHyperlink(link)
			end
		end
	end,
	shareQuest = function(value)
		if core.numPartyMembers > 0 and core.quests[value].complete[core.playerName] and core.quests[value].id and core.quests[value].sharable then
			local wasSelected = GetQuestLogSelection()
			SelectQuestLogEntry(core.quests[value].id)
			QuestLogPushQuest()
			SelectQuestLogEntry(wasSelected)
		end
	end,
	trackQuest = function(value)
		module:toggleTrackedQuest(nil, value)
	end,
	abandonQuest = function(value)
		if core.quests[value].complete[core.playerName] and core.quests[value].id then
			local wasSelected = GetQuestLogSelection()
			SelectQuestLogEntry(core.quests[value].id)
			SetAbandonQuest()
			local items = GetAbandonQuestItems()
			if items then
				StaticPopup_Hide("ABANDON_QUEST")
				StaticPopup_Show("ABANDON_QUEST_WITH_ITEMS", GetAbandonQuestName(), items)
			else
				StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS")
				StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName())
			end
			SelectQuestLogEntry(wasSelected)
		end
	end,
	displayQuest = function(value)
		core:displayQuestInformations(value)
	end,
	displayQuestInfo = function(value, value2)
		if value2 then
			core.cbh:Fire("toggleQuestInfo", value, value2)
		end
	end,
	displayCQIStartNPC = function(value)
		if Cartographer_QuestInfo then
			local q = Cartographer_QuestInfo:GetQuest(value)
			if q and q.start_npc then
				data.quest = q.title_full
				data.obj = nil
				data.npc = q.start_npc
				Cartographer_QuestInfo:OnButtonClick(nil, "start", "LeftButton", data)
			end
		end
	end,
	displayCQIEndNPC = function(value)
		if Cartographer_QuestInfo then
			local q = Cartographer_QuestInfo:GetQuest(value)
			if q and q.end_npc then
				data.quest = q.title_full
				data.obj = nil
				data.npc = q.end_npc
				Cartographer_QuestInfo:OnButtonClick(nil, "end", "LeftButton", data)
			end
		end
	end,
	displayCQIObjective = function(value, value2)
		if value2 and Cartographer_QuestInfo then
			local q = Cartographer_QuestInfo:GetQuest(value)
			if q and q.objs then
				for _, obj in pairs(q.objs) do
					if obj.title:find(core.questsObjectives[value][value2].desc, 1, true) then
						data.quest = q.title_full
						data.obj = obj
						data.npc = nil
					end
				end
				if data.obj ~= nil then
					Cartographer_QuestInfo:OnButtonClick(nil, "obj", "LeftButton", data)
				end
			end
		end
	end,
	displayMMObjective = function(value, value2)
		if value2 and MobMapDotParentFrame and MobMap_ParseQuestObjective then
			MobMap_ParseQuestObjective(core.questsObjectives[value][value2].desc)
		end
	end,
}

local questsOnMouseDown = function(cell, button)
	if not cell.arg then return end
	local success, category, qtype, value, value2, anchor = module:Deserialize(cell.arg)
	if not success then return end
	if category == "achievements" then
		achievementTracker:achievementOnMouseDown(cell, button)
	elseif category == "quests" then
		if qtype == "zone" then
			if chdb.zoneDisplay[value] == 1 then
				chdb.zoneDisplay[value] = 0
			else
				chdb.zoneDisplay[value] = 1
			end
			module:updateQuestsTracker()
		elseif qtype == "quest" then
			if button == "RightButton" then
				if IsShiftKeyDown() then
					interactivity[prdb.mouseRightShift](value, value2)
				elseif IsControlKeyDown() then
					interactivity[prdb.mouseRightControl](value, value2)
				elseif IsAltKeyDown() then
					interactivity[prdb.mouseRightAlt](value, value2)
				else
					interactivity[prdb.mouseRight](value, value2)
				end
			elseif button == "LeftButton" then
				if IsShiftKeyDown() then
					interactivity[prdb.mouseLeftShift](value, value2)
				elseif IsControlKeyDown() then
					interactivity[prdb.mouseLeftControl](value, value2)
				elseif IsAltKeyDown() then
					interactivity[prdb.mouseLeftAlt](value, value2)
				else
					interactivity[prdb.mouseLeft](value, value2)
				end
			elseif button == "MiddleButton" then
				if IsShiftKeyDown() then
					interactivity[prdb.mouseMiddleShift](value, value2)
				elseif IsControlKeyDown() then
					interactivity[prdb.mouseMiddleControl](value, value2)
				elseif IsAltKeyDown() then
					interactivity[prdb.mouseMiddleAlt](value, value2)
				else
					interactivity[prdb.mouseMiddle](value, value2)
				end
			end
		elseif qtype == "questItem" then
			core:generateSpecialItemMacro(nil, value)
		end
	end
end
local questsOnEnter = function(cell)
	if not cell.arg then return end
	local success, category, qtype, value, value2, anchor = module:Deserialize(cell.arg)
	if not success then return end
	if category == "achievements" then
		achievementTracker:achievementOnEnter(cell)
	elseif category == "quests" then
		if prdb.overTooltip and qtype == "quest" then
			if value and value2 and core.questsObjectives[value] and core.questsObjectives[value][value2].qType == "object" and core.questsItems[core.questsObjectives[value][value2].desc].mobDrop then
				GameTooltip:SetOwner(tracker, anchor)
				GameTooltip:AddLine(LQTr["|cffffffffDrop from:|r"])
				for mobName in pairs(core.questsItems[core.questsObjectives[value][value2].desc].mobDrop) do
					local zones = ""
					for zone in pairs(core.questsMobsItems[mobName].mobLoc) do
						zones = zones == "" and zone or format("%s, %s", zones, zone)
					end
					if core.questsMobsItems[mobName].items[core.questsObjectives[value][value2].desc].dropRate then
						GameTooltip:AddDoubleLine(format(LQTr["  |cffffffff%s in|r |cff00ff00%s|r"], mobName, zones), format("|cffffffff%s%%|r", core.questsMobsItems[mobName].items[core.questsObjectives[value][value2].desc].dropRate))
					else
						GameTooltip:AddLine(format(LQTr["  |cffffffff%s in|r |cff00ff00%s|r"], mobName, zones))
					end
				end
				GameTooltip:Show()
			elseif core.quests[value] and core.quests[value].id then
				local link = GetQuestLink(core.quests[value].id)
				if link then
					GameTooltip:SetOwner(tracker, anchor)
					GameTooltip:SetHyperlink(link)
				end
			end
		end
	end
end
local questsOnLeave = function(cell)
	GameTooltip:Hide()
end

local cProvider, cPrototype, basecCellPrototype = QTIP:CreateCellProvider(QTIP.LabelProvider)
function cPrototype:SetupCell(tooltip, value, justification, font, disableInteraction, arg, ...)
	local width, height = basecCellPrototype.SetupCell(self, tooltip, value, justification, font, ...)
	if not disableInteraction then
		self:EnableMouse(true)
		self.arg = arg
		self:SetScript("OnEnter", questsOnEnter)
		self:SetScript("OnLeave", questsOnLeave)
		self:SetScript("OnMouseDown", questsOnMouseDown)
	else
		self:EnableMouse(false)
		self.arg = nil
		self:SetScript("OnEnter", nil)
		self:SetScript("OnLeave", nil)
		self:SetScript("OnMouseDown", nil)
	end
	return width, height
end

function cPrototype:ReleaseCell()
	self:EnableMouse(false)
	self.arg = nil
	self:SetScript("OnEnter", nil)
	self:SetScript("OnLeave", nil)
	self:SetScript("OnMouseDown", nil)
end

local function displayQuests()
	if (not prdb.toggleTracker) or (numTQ == 0) or (hideInCombat) or (not prdb.onlyCurrentZone and ((prdb.hideCompletedQuest == 1 and ((prdb.limitQuestLevel and numTQ_BML == core.numTQ_PlQC_BML) or (not prdb.limitQuestLevel and numTQ == core.numTQ_PlQC))) or (prdb.hideCompletedQuest == 2 and ((prdb.limitQuestLevel and numTQ_BML == core.numTQ_PaQC_BML) or (not prdb.limitQuestLevel and numTQ == core.numTQ_PaQC))) or (prdb.hideCompletedQuest == 3 and (prdb.limitQuestLevel and core.numTQ_BML == 0)))) or (prdb.onlyCurrentZone and ((not core.questsByZone[currentZone]) or (core.questsByZone[currentZone].numTQ == 0) or (prdb.hideCompletedQuest == 1 and ((prdb.limitQuestLevel and core.questsByZone[currentZone].numTQ_BML == core.questsByZone[currentZone].numTQ_PlQC_BML) or (not prdb.limitQuestLevel and core.questsByZone[currentZone].numTQ == core.questsByZone[currentZone].numTQ_PlQC))) or (prdb.hideCompletedQuest == 2 and ((prdb.limitQuestLevel and core.questsByZone[currentZone].numTQ_BML == core.questsByZone[currentZone].numTQ_PaQC_BML) or (not prdb.limitQuestLevel and core.questsByZone[currentZone].numTQ == core.questsByZone[currentZone].numTQ_PaQC))) or (prdb.hideCompletedQuest == 3 and prdb.limitQuestLevel and core.questsByZone[currentZone].numTQ_BML == 0))) then
		return false
	else
		return true
	end
end

local function updateQuests(numColumns, disableInteraction)
	local tempLine
	if prdb.displayHeader then
		for zone, zData in pairs(core.questsByZone) do
			if ((prdb.onlyCurrentZone and zone == currentZone) or not prdb.onlyCurrentZone) and ((prdb.hideCompletedQuest == 1 and ((prdb.limitQuestLevel and zData.numTQ_BML > zData.numTQ_PlQC_BML) or (not prdb.limitQuestLevel and zData.numTQ > zData.numTQ_PlQC))) or (prdb.hideCompletedQuest == 2 and ((prdb.limitQuestLevel and zData.numTQ_BML > zData.numTQ_PaQC_BML) or (not prdb.limitQuestLevel and zData.numTQ > zData.numTQ_PaQC))) or (prdb.hideCompletedQuest == 3 and ((prdb.limitQuestLevel and zData.numTQ_BML > 0) or (not prdb.limitQuestLevel and zData.numTQ > 0)))) then
				chdb.zoneDisplay[zone] = chdb.zoneDisplay[zone] or 1
				tempLine = tracker:AddLine()
				tracker:SetCell(tempLine, 1, format("|TInterface\\Buttons\\UI-%sButton-Up:%d|t |cff%s%s|r |cff%s%s|r", chdb.zoneDisplay[zone] == 0 and "Plus" or "Minus", prdb.collapseButtonSize, prdb.colorizeHeaderName and zData.color or prdb.defaultHeaderNameColor, zone, prdb.colorizeHeaderLevel and zData.color or prdb.defaultHeaderLevelColor, prdb.hideZoneLevelRange and "" or zData.level ), hTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "zone", zone, nil, trackerAnchor), 0, 0, prdb.maxWidth)
				if chdb.zoneDisplay[zone] == 1 then
					for uid in pairs(zData.quests) do
						if chdb.questsTracked[uid] and ((prdb.hideCompletedQuest == 3) or (prdb.hideCompletedQuest == 2 and core.quests[uid].partyComplete == 0) or (prdb.hideCompletedQuest == 1 and core.quests[uid].playerComplete == 0)) and ((prdb.limitQuestLevel and (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff))) or not prdb.limitQuestLevel) then
							tempLine = tracker:AddLine()
							if prdb.showQuestItemIcon and core.quests[uid].specialItemIcon then
								tracker:SetCell(tempLine, 1, format("|cff%s%s|r |cff%s%s|r |T%s:%d|t %s", prdb.colorizeQuestLevel and core.quests[uid].color or prdb.defaultQuestLevelColor, core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or prdb.defaultQuestNameColor, core.quests[uid].title, core.quests[uid].specialItemIcon, prdb.questItemIconSize, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), qTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), prdb.questsCellPadding, 0, prdb.maxWidth)
							else
								tracker:SetCell(tempLine, 1, format("|cff%s%s|r |cff%s%s|r %s", prdb.colorizeQuestLevel and core.quests[uid].color or prdb.defaultQuestLevelColor, core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or prdb.defaultQuestNameColor, core.quests[uid].title, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), qTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), prdb.questsCellPadding, 0, prdb.maxWidth)
							end
							tracker:SetCell(tempLine, numColumns, core.quests[uid].pqStatus, qTrFontTQ2, "RIGHT", 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
							if core.questsObjectives[uid] then
								for objective, qoData in pairs(core.questsObjectives[uid]) do
									if prdb.hideCompletedObjective == 3 or (prdb.hideCompletedObjective == 2 and qoData.partyComplete == 0) or (prdb.hideCompletedObjective == 1 and qoData.playerComplete == 0) then
										tempLine = tracker:AddLine()
										tracker:SetCell(tempLine, 1, format("|cff%s%s|r", prdb.colorizeQuestObjective and qoData.qoColor or prdb.defaultQuestObjectiveColor, qoData.desc), qoTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, objective, trackerAnchor), prdb.objectivesCellPadding, 0, prdb.maxWidth)
										tracker:SetCell(tempLine, numColumns, prdb.colorizeQuestObjectiveCounter and qoData.qoStatus or qoData.qoStatusNC, qoTrFontTQ2, "RIGHT", 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, objective, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
									end
								end
							else
								tempLine = tracker:AddLine()
								tracker:SetCell(tempLine, 1, core.quests[uid].objective, qoTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), prdb.objectivesCellPadding, prdb.rightCellPadding, prdb.maxWidth)
							end
						end
					end
				end
			end
		end
	else
		for uid in pairs(chdb.questsTracked) do
			if uid and core.quests[uid] then
				if ((prdb.onlyCurrentZone and core.quests[uid].zone == currentZone) or not prdb.onlyCurrentZone) and ((prdb.hideCompletedQuest == 1 and core.quests[uid].playerComplete == 0) or (prdb.hideCompletedQuest == 2 and core.quests[uid].partyComplete == 0) or (prdb.hideCompletedQuest == 3)) and ((prdb.limitQuestLevel and (core.quests[uid].weightedLevel - core.playerLevel <= tonumber(core.db.profile.quests.maxLevelDiff))) or not prdb.limitQuestLevel) then
					tempLine = tracker:AddLine()
					if prdb.showQuestItemIcon and core.quests[uid].specialItemIcon then
						tracker:SetCell(tempLine, 1, format("|cff%s%s|r |cff%s%s|r |T%s:%d|t %s", prdb.colorizeQuestLevel and core.quests[uid].color or prdb.defaultQuestLevelColor, core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or prdb.defaultQuestNameColor, core.quests[uid].title, core.quests[uid].specialItemIcon, prdb.questItemIconSize, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), qTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), 0, 0, prdb.maxWidth)
					else
						tracker:SetCell(tempLine, 1, format("|cff%s%s|r |cff%s%s|r %s", prdb.colorizeQuestLevel and core.quests[uid].color or prdb.defaultQuestLevelColor, core.quests[uid].tag, prdb.colorizeQuestName and core.quests[uid].color or prdb.defaultQuestNameColor, core.quests[uid].title, core.quests[uid].playersOnQuest > 1 and format("<%d>", core.quests[uid].playersOnQuest) or ""), qTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), 0, 0, prdb.maxWidth)
					end
					tracker:SetCell(tempLine, numColumns, core.quests[uid].pqStatus, qTrFontTQ2, "RIGHT", 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
					if core.questsObjectives[uid] then
						for objective, qoData in pairs(core.questsObjectives[uid]) do
							if prdb.hideCompletedObjective == 3 or (prdb.hideCompletedObjective == 2 and qoData.partyComplete == 0) or (prdb.hideCompletedObjective == 1 and qoData.playerComplete == 0) then
								tempLine = tracker:AddLine()
								tracker:SetCell(tempLine, 1, format("|cff%s%s|r", prdb.colorizeQuestObjective and qoData.qoColor or prdb.defaultQuestObjectiveColor, qoData.desc), qoTrFontTQ2, "LEFT", numColumns - 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, objective, trackerAnchor), prdb.objectivesCellPadding, 0, prdb.maxWidth)
								tracker:SetCell(tempLine, numColumns, prdb.colorizeQuestObjectiveCounter and qoData.qoStatus or qoData.qoStatusNC, qoTrFontTQ2, "RIGHT", 1, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, objective, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
							end
						end
					else
						tempLine = tracker:AddLine()
						tracker:SetCell(tempLine, 1, core.quests[uid].objective, qoTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "quest", uid, nil, trackerAnchor), prdb.objectivesCellPadding, prdb.rightCellPadding, prdb.maxWidth)
					end
				end
			end
		end
	end
	if prdb.showItemsStartingQuest and core.numItemsStartingQuest > 0 then
		local zone = LQTr["Items starting a quest"]
		tempLine = tracker:AddLine()
		chdb.zoneDisplay[zone] = chdb.zoneDisplay[zone] or 1
		tracker:SetCell(tempLine, 1, format("|TInterface\\Buttons\\UI-%sButton-Up:%d|t |cff%s%s|r", chdb.zoneDisplay[zone] == 0 and "Plus" or "Minus", prdb.collapseButtonSize, prdb.colorizeHeaderName and "ffffff" or prdb.defaultHeaderNameColor, zone), hTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "zone", zone, nil, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
		if chdb.zoneDisplay[zone] == 1 then
			for item, itemData in pairs(core.itemStartingQuest) do
				tempLine = tracker:AddLine()
				tracker:SetCell(tempLine, 1, format("|T%s:%d|t |cff%s%s|r", itemData.icon, prdb.questItemIconSize, itemData.color, itemData.name), qTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "questItem", item, nil, trackerAnchor), prdb.questsCellPadding, prdb.rightCellPadding, prdb.maxWidth)
			end
		end
	end
	if prdb.showUsableQuestItemsNotAttached and core.numUsableQuestItems > 0 then
		local zone = LQTr["Usable quest items not linked to a quest"]
		tempLine = tracker:AddLine()
		chdb.zoneDisplay[zone] = chdb.zoneDisplay[zone] or 1
		tracker:SetCell(tempLine, 1, format("|TInterface\\Buttons\\UI-%sButton-Up:%d|t |cff%s%s|r", chdb.zoneDisplay[zone] == 0 and "Plus" or "Minus", prdb.collapseButtonSize, prdb.colorizeHeaderName and "ffffff" or prdb.defaultHeaderNameColor, zone), hTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "zone", zone, nil, trackerAnchor), 0, prdb.rightCellPadding, prdb.maxWidth)
		if chdb.zoneDisplay[zone] == 1 then
			for item, itemData in pairs(core.usableQuestItems) do
				tempLine = tracker:AddLine()
				tracker:SetCell(tempLine, 1, format("|T%s:%d|t |cff%s%s|r", itemData.icon, prdb.questItemIconSize, itemData.color, itemData.name), qTrFontTQ2, "LEFT", numColumns, cProvider, disableInteraction, module:Serialize("quests", "questItem", item, nil, trackerAnchor), prdb.questsCellPadding, prdb.rightCellPadding, prdb.maxWidth)
			end
		end
	end
end

function module:updateQuestsTracker()
	local numColumns
	if core:getModuleEnabled("achievementTracker") and achievementTracker.db.profile.showInQuestTracker then
		local achievementDisplayed, numColumns = achievementTracker:displayAchievement()
		local questDisplayed = displayQuests()
		if achievementDisplayed and (achievementTracker.db.profile.positionInQuestTracker == 1 or not questDisplayed) then
			tracker = QTIP:Acquire("tomQuest2Tracker", numColumns)
			tracker:Clear()
			tracker:SetBackdropColor(prdb.backDropColor[1], prdb.backDropColor[2], prdb.backDropColor[3], prdb.backDropColor[4])
			tracker:SetBackdropBorderColor(prdb.borderColor[1], prdb.borderColor[2], prdb.borderColor[3], prdb.borderColor[4])
			tracker:SetScale(prdb.scale)
			tracker:SetFrameStrata("BACKGROUND")
			achievementTracker:updateAchievement(tracker, trackerAnchor, prdb.disableInteraction)
			if questDisplayed then
				updateQuests(numColumns)
			end
		elseif questDisplayed then
			tracker = QTIP:Acquire("tomQuest2Tracker", numColumns)
			tracker:Clear()
			tracker:SetBackdropColor(prdb.backDropColor[1], prdb.backDropColor[2], prdb.backDropColor[3], prdb.backDropColor[4])
			tracker:SetBackdropBorderColor(prdb.borderColor[1], prdb.borderColor[2], prdb.borderColor[3], prdb.borderColor[4])
			tracker:SetScale(prdb.scale)
			tracker:SetFrameStrata("BACKGROUND")
			updateQuests(numColumns)
			if achievementDisplayed then
				achievementTracker:updateAchievement(tracker, trackerAnchor, prdb.disableInteraction)
			end
		else
			return
		end
	elseif displayQuests() then
		tracker = QTIP:Acquire("tomQuest2Tracker", 2)
		tracker:Clear()
		tracker:SetBackdropColor(prdb.backDropColor[1], prdb.backDropColor[2], prdb.backDropColor[3], prdb.backDropColor[4])
		tracker:SetBackdropBorderColor(prdb.borderColor[1], prdb.borderColor[2], prdb.borderColor[3], prdb.borderColor[4])
		tracker:SetScale(prdb.scale)
		tracker:SetFrameStrata("BACKGROUND")
		updateQuests(2, prdb.disableInteraction)
	else
		QTIP:Release(tracker)
		return
	end
	tracker:ClearAllPoints()
	if prdb.showAnchor then
		tracker:SetPoint(anchorSetPoint[prdb.anchorPosition].point, anchor, anchorSetPoint[prdb.anchorPosition].relPoint)
	else
		tracker:SetPoint(anchorSetPoint[prdb.anchorPosition].point, anchor, anchorSetPoint[prdb.anchorPosition].relPoint, anchorSetPoint[prdb.anchorPosition].x, anchorSetPoint[prdb.anchorPosition].y)
	end
	if not prdb.disableInteraction then
		tracker:UpdateScrolling(prdb.hasMaxHeight and prdb.maxHeight or nil)
		tracker:EnableMouse(true)
	else
		tracker:EnableMouse(false)
	end
	tracker:Show()
end

--------------------------
--- Keybinding globals ---
--------------------------
BINDING_NAME_TQ2QUESTTRACKERTOGGLE = LQTr["Quests Tracker"]

function tQ2toggleQuestTracker()
	prdb.toggleTracker = not prdb.toggleTracker
	if not prdb.toggleTracker then
		QTIP:Release(tracker)
		tracker = nil
	else
		module:updateQuestsTracker()
	end
end