-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("soundNotifications", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LSN = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-SoundNotifications")
local LSM = LibStub("LibSharedMedia-3.0")

---------------------------------------
--- Shared Media Sound Registration ---
---------------------------------------
LSM:Register("sound", "Peon Ready", [[Sound\Creature\Peon\PeonReady1.wav]])
LSM:Register("sound", "Work Done", [[Interface\Addons\tomQuest2\media\Peasant_work_done.mp3]])
LSM:Register("sound", "Thrumm", [[Sound\Interface\igPlayerBind.wav]])
LSM:Register("sound", "More work?", [[Sound\Creature\Peasant\PeasantWhat3.wav]])
LSM:Register("sound", "Work complete!", [[Sound\Creature\Peon\PeonBuildingComplete1.wav]])
LSM:Register("sound", "Quest Added", [[Sound\interface\iQuestActivate.wav]])
LSM:Register("sound", "Quest Completed", [[Sound\interface\iQuestComplete.wav]])
LSM:Register("sound", "Quest Failed", [[Sound\interface\igQuestFailed.wav]])
LSM:Register("sound", "Auction Window Open", [[Sound\interface\AuctionWindowOpen.wav]])
LSM:Register("sound", "Auction Window Close", [[Sound\interface\AuctionWindowClose.wav]])
LSM:Register("sound", "Ready Check", [[Sound\interface\levelup2.wav]])
LSM:Register("sound", "Level Up", [[Sound\interface\levelup.wav]])

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local PlaySoundFile = PlaySoundFile
local PlaySound = PlaySound

-----------------
--- Variables ---
-----------------
module.modName = LSN["Sound Notifications"]
local prdb, options

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		playerObjectiveComplete = true,
		playerQuestComplete = true,
		playerQuestFailed = true,
		soundQuestComplete = "Work Done",
		soundQuestFailed = "Thrumm",
		soundObjectiveComplete = "Peon Ready",
	},
}

---------------
--- Options ---
---------------
local optGetter, optSetter, hidePOC, hidePQC, hidePQF
do
	function optGetter(info)
		return prdb[info[#info]]
	end
	function optSetter(info, value)
		prdb[info[#info]] = value
	end
	function hidePOC()
		if prdb.playerObjectiveComplete then
			return false
		else
			return true
		end
	end
	function hidePQC()
		if prdb.playerQuestComplete then
			return false
		else
			return true
		end
	end
	function hidePQF()
		if prdb.playerQuestFailed then
			return false
		else
			return true
		end
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = optGetter,
			set = optSetter,
			args = {
				playerObjectiveComplete = {
					name = LSN["Quest objective complete notification"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				playerQuestComplete = {
					name = LSN["Quest complete notification"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				playerQuestFailed = {
					name = LSN["Quest failed notification"],
					type = "toggle",
					width = "full",
					order = 12,
				},
				soundObjectiveComplete = {
					name = LSN["Quest objective complete sound"],
					hidden = hidePOC,
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					width = "full",
					order = 13,
				},
				soundQuestComplete = {
					name = LSN["Quest complete sound"],
					hidden = hidePQC,
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					width = "full",
					order = 14,
				},
				soundQuestFailed = {
					name = LSN["Quest failed sound"],
					hidden = hidePQF,
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					width = "full",
					order = 15,
				},
			},
		}
	end
	return options
end

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("soundNotifications", defaults, "Default")
	prdb = self.db.profile
	self:SetEnabledState(core:getModuleEnabled("soundNotifications"))
	core:registerModuleOptions("soundNotifications", getOptions, module.modName)
end

function module:OnEnable()
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
	self:Refresh()
end

function module:Refresh()
	prdb = self.db.profile
	if not self:IsEnabled() then return end
end

local function Play(sound)
	local soundPlayed = PlaySoundFile(sound)
	if not soundPlayed then
		PlaySound(sound)
	end
end

---------------------------
--- Main Module Program ---
---------------------------
function module:playerQuestObjectiveProgress(event, firstRun, commHandler, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and prdb.playerObjectiveComplete and not firstRun and core.quests[uid].complete[core.playerName] == 0 then
		Play(LSM:Fetch("sound", prdb.soundObjectiveComplete))
	end
end


function module:playerQuestComplete(event, firstRun, commHandler, uid)
	if prdb.playerQuestComplete and not firstRun then
		Play(LSM:Fetch("sound", prdb.soundQuestComplete))
	end
end

function module:playerQuestFailed(event, firstRun, commHandler, uid)
	if prdb.playerQuestFailed and not firstRun then
		Play(LSM:Fetch("sound", prdb.soundQuestFailed))
	end
end